/***************************************************************************
 * Copyright (c)  2014  Zephyr Software LLC. All rights reserved.
 *
 * This software is furnished under a license and/or other restrictive
 * terms and may be used and copied only in accordance with such terms
 * and the inclusion of the above copyright notice. This software or
 * any other copies thereof may not be provided or otherwise made
 * available to any other person without the express written consent
 * of an authorized representative of Zephyr Software LCC. Title to,
 * ownership of, and all rights in the software is retained by
 * Zephyr Software LCC.
 *
 * Zephyr Software LLC. Proprietary Information
 *
 * Unless otherwise specified, the information contained in this
 * directory, following this legend, and/or referenced herein is
 * Zephyr Software LLC. (Zephyr) Proprietary Information. 
 *
 * CONTACT
 *
 * For technical assistance, contact Zephyr Software LCC. at:
 *      
 *
 * Zephyr Software, LLC
 * 2040 Tremont Rd
 * Charlottesville, VA 22911
 *
 * E-mail: jwd@zephyr-software.com
 **************************************************************************/

#ifndef zipr_sdk_memory_space_h
#define zipr_sdk_memory_space_h


namespace Zipr_SDK
{

typedef std::set<Range_t, Range_tCompare> RangeSet_t;

// a memory space _is_ a map of range addres to char, with additional functionality.
class MemorySpace_t : public std::map<RangeAddress_t,char>
{
	public:

		// range operatations
		virtual void SplitFreeRange(RangeAddress_t addr)=0;
		virtual void SplitFreeRange(Range_t range)=0;
		virtual void MergeFreeRange(RangeAddress_t addr)=0;
		virtual void MergeFreeRange(Range_t range)=0;
		virtual RangeSet_t::iterator FindFreeRange(RangeAddress_t addr)=0;
		virtual Range_t GetFreeRange(int size)=0;
		virtual std::list<Range_t> GetFreeRanges(size_t size = 0) = 0;
		virtual std::pair<RangeSet_t::const_iterator,RangeSet_t::const_iterator>
			GetNearbyFreeRanges(const RangeAddress_t hint, size_t count = 0) = 0;
		virtual void AddFreeRange(Range_t newRange)=0;
		virtual void RemoveFreeRange(Range_t newRange)=0;
		virtual Range_t GetLargeRange(void)=0;

		// queries about free areas.
		virtual bool AreBytesFree(RangeAddress_t addr, int num_bytes)=0;
		virtual bool IsByteFree(RangeAddress_t addr)=0;
		virtual bool IsValidRange(RangeSet_t::iterator it)=0;
		virtual int GetRangeCount()=0;
		virtual void PrintMemorySpace(std::ostream &out)=0;

		virtual void PlopBytes(RangeAddress_t addr, const char the_byte[], int num)=0;
		virtual void PlopByte(RangeAddress_t addr, char the_byte)=0;
		virtual void PlopJump(RangeAddress_t addr)=0;
		virtual RangeAddress_t GetMinPlopped() const =0;
		virtual RangeAddress_t GetMaxPlopped() const =0;

};

}
#endif

